#!/usr/bin/python
#	
#	updateSite.py - Part of PubTal
#
#	Usage: updateSite.py site.config [content-dir | content-file] [...]
#
#	Copyright (c) 2003 Colin Stewart (http://www.owlfish.com/)
#	All rights reserved.
#		
#	Redistribution and use in source and binary forms, with or without
#	modification, are permitted provided that the following conditions
#	are met:
#	1. Redistributions of source code must retain the above copyright
#	   notice, this list of conditions and the following disclaimer.
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#	3. The name of the author may not be used to endorse or promote products
#	   derived from this software without specific prior written permission.
#		
#	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#	
#	If you make any bug fixes or feature enhancements please let me know!
#

try:
	import logging
	logging.basicConfig()
	root = logging.getLogger()
	#root.setLevel (logging.DEBUG)
	root.setLevel (logging.INFO)
	talLogger = logging.getLogger ("simpleTAL")
	talLogger.setLevel (logging.INFO)
	talesLogger = logging.getLogger ("simpleTALES")
	talesLogger.setLevel (logging.INFO)

except:
	from simpletal import DummyLogger as logging
	
import sys, os.path, os, stat, copy, time, re, codecs
import pubtal

from pubtal import ContentToHTMLConverter, ArticleContent, ConfigurationParser

from simpletal import simpleTAL, simpleTALES, simpleTALUtils

class SiteConfig:
	def __init__ (self, configFile):
		self.log = logging.getLogger ("SiteConfig")
		self.fileTypeMap = {}
		self.fileMap = {}
		self.directoryMap = {}
		# List of regular expressions that match files to be ignored
		self.ignoreFilters = []
		self.baseDir = os.path.abspath (os.path.split (configFile)[0])
		
		# Defaults
		self.setCharacterSet ('ISO8859-15')
		self.contentDir = os.path.join (self.baseDir, 'content')
		self.destDir = os.path.join (self.baseDir, 'dest')
		self.templateDir = os.path.join (self.baseDir, 'template')
		
		defaultConfig = PageConfigItem()
		defaultConfig.setContentType ('HTMLText')
		self.fileTypeMap ['txt'] = defaultConfig
		
		self.readConfig(configFile)
		
	def getContentDir (self):
		return self.contentDir
		
	def getDestinationDir (self):
		return self.destDir
		
	def getIgnoreFilters (self):
		return self.ignoreFilters
		
	def getCharacterSet (self):
		return self.characterSet
		
	def setCharacterSet (self, charSet):
		self.characterSet = charSet
		
	def getPage (self, contentPath):
		page = Page (contentPath, self.contentDir, self.destDir, self.templateDir)
		# Look for file type first
		configItem = self.fileTypeMap.get (contentPath [contentPath.rfind ('.')+1:])
		if (configItem is not None):
			configItem.updatePage (page)
		
		# Now work through directory map
		dirConfigItems = []
		head, tail = os.path.split (contentPath)
		while (tail != ''):
			self.log.debug ("Looking for directory configuration at: %s" % head)
			configItem = self.directoryMap.get (head, None)
			if (configItem is not None):
				dirConfigItems.insert (0,configItem)
			if (self.contentDir.startswith (head)):
				# We have reached the top of the content dir, so stop now
				tail = ''
			else:
				head, tail = os.path.split (head)
			
		for confItem in dirConfigItems:
			self.log.debug ("Updating config using %s" % str (confItem))
			confItem.updatePage (page)
		
		# Now look for this specific file
		configItem = self.fileMap.get (contentPath)
		if (configItem is not None):
			configItem.updatePage (page)
			
		return page
		
	def readConfig (self, configFile):
		parser = ConfigurationParser.ConfigurationParser()
		parser.addHandler ('SiteConfig', self)
		parser.addHandler ('FileType', self)
		parser.addHandler ('Directory', self)
		parser.addHandler ('File', self)
		self.currentDirective = None
		self.currentConfigItem = None
		confFile = open (configFile, 'r')
		parser.parse(confFile)
		confFile.close()
		
		for fileType in self.fileTypeMap.keys():
			self.log.debug ("FileType %s has Configuration: %s" % (fileType, str (self.fileTypeMap [fileType])))
		for dir in self.directoryMap.keys():
			self.log.debug ("Directory %s has Configuration: %s" % (dir, str (self.directoryMap [dir])))
		for file in self.fileMap.keys():
			self.log.debug ("File %s has Configuration: %s" % (file, str (self.fileMap [file])))
		
	def startDirective (self, directive, options):
		self.currentDirective = directive
		if (directive == 'FILETYPE'):
			self.currentConfigItem = PageConfigItem()
			self.fileTypeMap [options] = self.currentConfigItem
		elif (directive == 'DIRECTORY'):
			self.currentConfigItem = PageConfigItem()
			targetDir = os.path.join (self.contentDir, options)
			if (targetDir.endswith (os.sep)):
				targetDir = targetDir [:-1]
			self.directoryMap [targetDir] = self.currentConfigItem
		elif (directive == 'FILE'):
			self.currentConfigItem = PageConfigItem()
			self.fileMap [os.path.join (self.contentDir, options)] = self.currentConfigItem
		else:
			self.currentConfigItem = None
		
	def endDirective (self, directive):
		self.currentConfigItem = None
		self.currentDirective = None
		
	def option (self, line):
		if (self.currentConfigItem is not None):
			if (line.lower().startswith ('template')):
				template = line [line.find (' ')+1:]
				self.currentConfigItem.setTemplate (os.path.join (self.templateDir, template))
			elif (line.lower().startswith ('content-type')):
				name = line [line.find (' ')+1:]
				self.currentConfigItem.setContentType (name)
			elif (line.lower().startswith ('macro')):
				mnStart = line.find (' ')
				mnEnd = line.find (' ', mnStart+1)
				macroName = line [mnStart+1:mnEnd]
				macro = line [mnEnd + 1:]
				self.currentConfigItem.addMacro (macroName, os.path.join (self.templateDir, macro))
			elif (line.lower().startswith ('header')):
				mnStart = line.find (' ')
				mnEnd = line.find (' ', mnStart+1)
				headerName = line [mnStart+1:mnEnd]
				header = line [mnEnd + 1:]
				self.currentConfigItem.addHeader (headerName, header)
			else:
				self.log.warn ("Option %s not supported" % line)
		elif (self.currentDirective == 'SITECONFIG'):
			if (line.lower().startswith ('content-dir')):
				self.contentDir = os.path.join (self.baseDir, line [line.find (' ')+1:])
			elif (line.lower().startswith ('template-dir')):
				self.templateDir = os.path.join (self.baseDir, line [line.find (' ')+1:])
			elif (line.lower().startswith ('dest-dir')):
				self.destDir = os.path.join (self.baseDir, line [line.find (' ')+1:])
			elif (line.lower().startswith ('ignore-filter')):
				filter = line [line.find (' ')+1:]
				self.log.info ("Adding filter of content to ignore: %s" % filter)
				self.ignoreFilters.append (re.compile (filter))
			elif (line.lower().startswith ('character-set')):
				self.setCharacterSet (line[line.find (' ') + 1:])
			else:
				self.log.warn ("SiteConfig Option %s not supported" % line)
		
class PageConfigItem:
	def __init__ (self):
		self.macros = {}
		self.template = None
		self.headers = {}
		self.contentType = None
		
	def updatePage (self, page):
		page.macros.update (self.macros)
		page.headers.update (self.headers)
		if (self.template is not None):
			page.template = self.template
		if (self.contentType is not None):
			page.contentType = self.contentType
			
	def addMacro (self, name, template):
		self.macros [name] = template
		
	def addHeader (self, name, header):
		self.headers [name] = header
		
	def setTemplate (self, template):
		self.template = template
		
	def setContentType (self, contentType):
		self.contentType = contentType
		
	def __str__ (self):
		desc = ""
		if (len (self.macros) > 0):
			desc += "Macros: %s\n" % str (self.macros)
		if (len (self.headers) > 0):
			desc += "Headers: %s\n" % str (self.headers)
		if (self.template is not None):
			desc += "Template: %s\n" % self.template
		if (self.contentType is not None):
			desc += "Content Type: %s" % self.contentType
		return desc

class PagePublisher:
	def __init__ (self, config):
		self.templateCache = simpleTALUtils.TemplateCache()
		self.config = config
		self.htmlTextPublisher = HTMLTextPagePublisher (self)
		self.log = logging.getLogger ("PagePublisher")
		
	def getTemplateCache (self):
		return self.templateCache, self.config.getCharacterSet()
		
	def getConfig (self):
		return self.config
		
	def publish (self, page):
		if (page.getContentType() == "HTMLText"):
			self.htmlTextPublisher.publish (page)
		else:
			self.log.warn ("Unsupported content type.")
		
class Page:
	def __init__ (self, source, contentDir, destDir, templateDir):
		# Things we need
		self.source = source
		self.macros = {}
		self.template = os.path.join (templateDir, 'template.html')
		self.headers = {}
		self.destDir = destDir
		self.contentType = None
		
		commonRoot = os.path.commonprefix ([contentDir, source])
		self.relativePath = source[len (commonRoot)+1:]
		# relativePath is of the form: [dir/]file
		depth = 0
		head, tail = os.path.split (self.relativePath)
		while (tail != ''):
			depth += 1
			head, tail = os.path.split (head)
		# We go to far, so knock one off!
		self.depth = depth - 1
		
	def getContentType (self):
		return self.contentType
		
	def getSource (self):
		return self.source
		
	def getRelativePath (self):
		return self.relativePath
		
	def getMarcos (self):
		return self.macros
		
	def getHeaders (self):
		return self.headers
		
	def getTemplate (self):
		return self.template
		
	def getDepthString (self):
		return "../"*self.depth
		
	def getModificationTime (self):
		""" Returns a tuple of the creation and modification date/time. """
		info = os.stat (self.source)
		return info[stat.ST_MTIME]
		
	def __str__ (self):
		return self.source
		
class ContextFunctions:
	def __init__ (self):
		self.currentPage = None
		self.isPage = simpleTALES.PathFunctionVariable (self.isCurrentPage)
		
	def setCurrentPage (self, page):
		self.currentPage = page
		
	def isCurrentPage (self, otherPage):
		if (self.currentPage == otherPage):
			return 1
		return 0
		
class HTMLTextPagePublisher:
	def __init__ (self, pagePublisher):
		self.pagePublisher = pagePublisher
		self.templateCache, self.characterSet = pagePublisher.getTemplateCache()
		self.characterSetCodec = codecs.lookup (self.characterSet)[1]
		self.destDir = pagePublisher.getConfig().getDestinationDir()
		self.converter = ContentToHTMLConverter.ContentToHTMLConverter()
		self.log = logging.getLogger ("HTMLTextPagePublisher")
		
		# Used in the Context
		self.ContextFunctions = ContextFunctions()
		self.pubTalInfo = {'version': pubtal.__version__
							,'url': "http://www.owlfish.com/software/PubTal/"
							,'linkText': """<p title="PubTal is a template driven web site publisher.">Made with <a href="http://www.owlfish.com/software/PubTal/">PubTal</a> %s</p>""" % pubtal.__version__
						}
		
	def publish (self, page):
		template = self.templateCache.getTemplate (page.template, inputEncoding=self.characterSet)
		context = simpleTALES.Context(allowPythonPath=1)
		headers, rawContent = self.getRawContent(page)
		content = self.converter.convertContent (rawContent)
		article = ArticleContent.ArticleContent ()
		article.setArticleBody (content)
		modTime = time.localtime (page.getModificationTime())
		article.modificationDate = modTime
		article.creationDate = modTime
		
		# Determine the destination for this page
		relativeDestPath = os.path.splitext (page.getRelativePath())[0] + '.html'
		destPath = os.path.join (self.destDir, relativeDestPath)
		
		self.ContextFunctions.setCurrentPage (relativeDestPath)
		context.addGlobal ('ispage', self.ContextFunctions.isPage)
		context.addGlobal ('pubtal', self.pubTalInfo)
		
		map = {	'lastModifiedDate': article.getRFC822Date()
				,'copyrightYear': article.getCopyrightYear()
				,'content': content
				,'rawContent': rawContent
				,'sourcePath': page.getRelativePath()
				,'absoluteSourcePath': page.getSource()
				,'destinationPath': relativeDestPath
				,'absoluteDestinationPath': destPath
				,'depth': page.getDepthString()
				}
		
		# Merge in configuration headers and per-page headers
		actualHeaders = page.getHeaders()
		actualHeaders.update (headers)
		map ['headers'] = actualHeaders
		self.log.debug (str (headers))
		context.addGlobal ('page', map)
		
		# Add macros to the context
		macroTemplates = {}
		macros = page.getMarcos()
		for macroName in macros.keys():
			mTemp = self.templateCache.getTemplate (macros [macroName], inputEncoding=self.characterSet)
			macroTemplates [macroName] = mTemp.macros
		
		context.addGlobal ('macros', macroTemplates)
		
		
		# Make directories if required.
		destDir = os.path.split (destPath)[0]
		if (not os.path.exists (destDir)):
			os.makedirs (destDir)
		dest = open (destPath, 'w')
		template.expand (context, dest)
		dest.close()
	
	def getRawContent (self, page):
		sourceFile = open (page.getSource(), 'r')
		readingHeaders = 1
		headers = {}
		while (readingHeaders):
			line = self.characterSetCodec (sourceFile.readline())[0]
			offSet = line.find (':')
			if (offSet > 0):
				headers [line[0:offSet]] = line[offSet + 1:].strip()
			else:
				readingHeaders = 0
		rawContent = self.characterSetCodec (sourceFile.read())[0]
		sourceFile.close()
		return (headers, rawContent)
					
class PathResolver:
	def __init__ (self, config):
		self.config = config
		self.currentContent = []
		self.log = logging.getLogger ('PathResolver')
		self.contentDir = config.getContentDir()
		self.ignoreFilters = config.getIgnoreFilters()
		
	def getPages (self, target):
		""" Returns a Page list
		
			target is either:
				None 		- Get all files
				List of files or dir paths.
		"""
		result = []
		
		if (target is None):
			self.log.info ("Building whole site.")
			targetList = [self.contentDir]
		else:
			targetList = []
			for t in target:
				targetList.append (os.path.abspath (t))
			
		self.log.debug ("Target path: %s" % str (targetList))
		for targetPath in targetList:
			self.log.debug ("Checking target path: %s" % targetPath)
			# Are we doing just one file or a dir?
			if (os.path.isfile (targetPath)):
				# Just get this entry
				result.append (self.config.getPage (targetPath))
			else:
				os.path.walk (targetPath, self.walkPaths, None)
				for content in self.currentContent:
					result.append (self.config.getPage (content))
		self.currentContent = []
		return result
		
	def walkPaths (self, arg, dirname, names):
		for name in names:
			self.log.debug ("Checking path %s for content." % os.path.join (dirname, name))
			realName = os.path.join (dirname, name)
			if (os.path.isfile (realName)):
				contentFile = 1
				for filter in self.ignoreFilters:
					if (filter.match (realName)):
						contentFile = 0
				if (contentFile):
					self.currentContent.append (realName)
				else:
					self.log.debug ("Ignoring path %s" % realName)

class UpdateSite:
	def __init__ (self, configFile, path=None):
		self.config = SiteConfig (configFile)
		self.pathResolver = PathResolver (self.config)
		self.target = path
		self.log = logging.getLogger ("UpdateSite")
		self.publisher = PagePublisher (self.config)
		
	def buildSite (self):
		pathsToBuild = self.pathResolver.getPages (self.target)
		self.log.debug ("Found paths: %s" % str (pathsToBuild))
		for aPage in pathsToBuild:
			self.log.info ("Publishing: %s" % str (aPage))
			self.publisher.publish (aPage)
		
if __name__ == '__main__':
	#logging.debug ("Arguments: " + str (sys.argv))
	if (len (sys.argv) < 2):
		print "Usage: %s configFile [paths]" % sys.argv[0]
		sys.exit (1)
		
	if (len (sys.argv) == 2):
		target = None
	else:
		target = sys.argv [2:]
			
	update = UpdateSite (sys.argv[1], target)
	update.buildSite()
