"""		Copyright (c) 2003 Colin Stewart (http://www.owlfish.com/)
		All rights reserved.
		
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions
		are met:
		1. Redistributions of source code must retain the above copyright
		   notice, this list of conditions and the following disclaimer.
		2. Redistributions in binary form must reproduce the above copyright
		   notice, this list of conditions and the following disclaimer in the
		   documentation and/or other materials provided with the distribution.
		3. The name of the author may not be used to endorse or promote products
		   derived from this software without specific prior written permission.
		
		THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
		IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
		OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
		IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
		INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
		NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
		DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
		THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
		THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import time, string
import xml.sax, xml.sax.saxutils

try:
	import logging
except:
	from simpletal import DummyLogger as logging

DATE_FORMAT="%d-%m-%Y"
TIME_FORMAT="%H:%M:%S"

class ArticleContent:
	def __init__ (self, existingArticle=None):
		self.log = logging.getLogger ("ArticleContent.ArticleContent")
		if (existingArticle is None):
			self.log.debug ("Creating a new article.")
			self.creationDate = time.localtime()
			self.modificationDate = self.creationDate
			self.articleTitle = u""
			self.articleBody = u""
		else:
			self.log.debug ("Loading existing article from: %s" % existingArticle)
			articleReader = ArticleContentReader (existingArticle, self)
			
	def setArticleBody (self, newBody):
		self.articleBody = newBody
		
	def setArticleTitle (self, newTitle):
		self.articleTitle = newTitle
		
	def getArticleBody (self):
		return self.articleBody
		
	def getArticleTitle (self):
		return self.articleTitle
		
	def getArticleCreationDate (self):
		return self.creationDate
		
	def getArticleModificationDate (self):
		return self.modificationDate
		
	def getDateShort (self):
		# Return the date as dd-mm-yyyy
		return time.strftime (DATE_FORMAT,self.getArticleCreationDate())
		
	def getDateLong (self):
		# Return the date as Monday, 11 November 2002
		dayStr = str (int (time.strftime ('%d', self.getArticleCreationDate())))
		return time.strftime ('%A, %%s %B %Y', self.getArticleCreationDate()) % dayStr
		
	def getRFC822Date (self):
		# Returns the date in RFC822 format
		return time.strftime ('%a, %d %b %Y %H:%M:%S %Z', self.getArticleCreationDate())
		
	def getTime24Hour (self):
		# Return the time as HH:MM in 24Hr format
		return time.strftime (TIME_FORMAT,self.getArticleCreationDate())
		
	def getCopyrightYear (self):
		""" Returns the current year """
		return time.strftime ('%Y')
		
	def getTime12Hour (self):
		# Return the time as hh:mm am/pm in 12 Hr format
		# We can't use %p because of a strange bug in 2.1
		dateTime = self.getArticleCreationDate()
		if (dateTime[3] >= 12):
			amPM = "pm"
			if (dateTime[3] == 12):
				hour = '12'
			else:
				hour = str (dateTime[3] - 12)
		else:
			amPM = "am"
			if (dateTime[3] == 0):
				hour = '12'
			else:
				hour = str (dateTime[3])
		return hour + ':' + string.zfill (str (dateTime[4]), 2) + " " + amPM
		
	def saveArticle (self, path):
		self.modificationDate = time.localtime()
		articleWriter = ArticleContentWriter (self)
		articleWriter.write (path)
	
class ArticleContentReader (xml.sax.handler.ContentHandler):
	def __init__ (self, file, article):
		self.log = logging.getLogger ("ArticleContent.ArticleContentReader")
		self.elementMap = {'creationDate': (u"", {})
											,'creationTime': (u"", {})
											,'modificationDate': (u"", {})
											,'modificationTime': (u"", {})
											,'title': (u"", {})
											,'body': (u"", {})}
		self.inArticle = 0
		self.articleVersion = 1.0
		self.currentData = u""
		self.currentAttributes = []
		self.log.debug ("Creating XML parser.")
		self.parser = xml.sax.parse (file, self)
		if (self.articleVersion != 1.1):
			raise "Unsupported article version."
		
		self.log.debug ("Determining creation date.")
		article.creationDate = time.strptime (self.getData ('creationDate')+self.getData ('creationTime')
																					, DATE_FORMAT + TIME_FORMAT)

		article.modificationDate = time.strptime (self.getData ('modificationDate')+self.getData ('modificationTime')
																							,DATE_FORMAT + TIME_FORMAT)																					
		self.log.debug ("Getting article title")
		article.articleTitle = self.getData ('title')
		self.log.debug ("Getting article body")
		article.articleBody = self.getData ('body')
		
	def getData (self, name, attName=None):
		data = self.elementMap [name]
		if (attName is not None):
			return data[1][attName]
		return data[0]
		
	def startElement (self, name, attributes):
		if (name == "article"):
			self.inArticle = 1
			self.articleVersion = float (attributes ["version"])
		if (self.inArticle and self.elementMap.has_key (name)):
			# Get the attributes
			self.currentAttributes = attributes
			self.currentData = u""
		
	def endElement (self, name):
		if (name == "article"):
			self.inArticle = 0
		if (self.inArticle and self.elementMap.has_key (name)):
			self.elementMap [name] = (self.currentData, self.currentAttributes)
		
	def characters (self, txt):
		self.currentData = self.currentData + txt
	
class ArticleContentWriter:
	"""<?xml version="1.0"?>
		<article version="1.0">
		  <creationDate>%s</creationDate>
		  <creationTime>%s</creationTime>
		  <modificationDate>%s</modificationDate>
		  <modificationTime>%s</modificationTime>
		  [<title>%s</title>]
		  <body><![CDATA[%s]]></body>
		</article>
		"""
	def __init__ (self, article):
		self.log = logging.getLogger ("ArticleContent.ArticleContentWriter")
		self.article = article
		
	def write (self, path):
		ENCODING = 'UTF-8'
		file = open (path, 'w')
		self.writer = xml.sax.saxutils.XMLGenerator (file, ENCODING)
		writer = self.writer
		
		writer.startDocument()
		writer.startElement ("article",{'version': '1.1'})
		
		createDate = self.article.getArticleCreationDate()
		modDate = self.article.getArticleModificationDate()
		self.log.debug ("Writing out date/time")
		self.element ("creationDate", time.strftime (DATE_FORMAT,createDate))
		self.element ("creationTime", time.strftime (TIME_FORMAT,createDate))
		self.element ("modificationDate", time.strftime (DATE_FORMAT,modDate))
		self.element ("modificationTime", time.strftime (TIME_FORMAT,modDate))
		
		self.log.debug ("Getting title")
		title = self.article.getArticleTitle()
		if (len (title.strip()) > 0):
			self.log.debug ("Found title - writing out")
			self.element ("title", title)
		
		self.log.debug ("Writing out body")	
		self.log.debug ("Article body type: " + str (type (self.article.getArticleBody())))
		self.element ("body", self.article.getArticleBody())
		
		writer.endElement ("article")
		writer.endDocument()
		del self.writer
		file.close()
		
	def element (self, name, value, attributes={}):
		self.writer.startElement (name, attributes)
		self.writer.characters (value)
		self.writer.endElement (name)
		self.writer.characters (u"\n")
						